"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _1 = require("./");
const n8n_workflow_1 = require("n8n-workflow");
const lodash_1 = require("lodash");
const path = require("path");
const requestPromise = require("request-promise-native");
const mmmagic_1 = require("mmmagic");
const magic = new mmmagic_1.Magic(mmmagic_1.MAGIC_MIME_TYPE);
async function prepareBinaryData(binaryData, filePath, mimeType) {
    if (!mimeType) {
        mimeType = await new Promise((resolve, reject) => {
            magic.detect(binaryData, (err, mimeType) => {
                if (err) {
                    return reject(err);
                }
                return resolve(mimeType);
            });
        });
    }
    const returnData = {
        mimeType,
        data: binaryData.toString(_1.BINARY_ENCODING)
    };
    if (filePath) {
        if (filePath.includes('?')) {
            filePath = filePath.split('?').shift();
        }
        const filePathParts = path.parse(filePath);
        returnData.fileName = filePathParts.base;
        const fileExtension = filePathParts.ext.slice(1);
        if (fileExtension) {
            returnData.fileExtension = fileExtension;
        }
    }
    return returnData;
}
exports.prepareBinaryData = prepareBinaryData;
function returnJsonArray(jsonData) {
    const returnData = [];
    if (!Array.isArray(jsonData)) {
        jsonData = [jsonData];
    }
    jsonData.forEach((data) => {
        returnData.push({ json: data });
    });
    return returnData;
}
exports.returnJsonArray = returnJsonArray;
function getCredentials(workflow, node, type, additionalData) {
    const nodeType = workflow.nodeTypes.getByName(node.type);
    if (nodeType === undefined) {
        throw new Error(`Node type "${node.type}" is not known so can not get credentials!`);
    }
    if (nodeType.description.credentials === undefined) {
        throw new Error(`Node type "${node.type}" does not have any credentials defined!`);
    }
    const nodeCredentialDescription = nodeType.description.credentials.find((credentialTypeDescription) => credentialTypeDescription.name === type);
    if (nodeCredentialDescription === undefined) {
        throw new Error(`Node type "${node.type}" does not have any credentials of type "${type}" defined!`);
    }
    if (n8n_workflow_1.NodeHelpers.displayParameter(node.parameters, nodeCredentialDescription, node.parameters) === false) {
        return undefined;
    }
    if (!node.credentials || !node.credentials[type]) {
        if (nodeCredentialDescription.required === true) {
            if (!node.credentials) {
                throw new Error('Node does not have any credentials set!');
            }
            if (!node.credentials[type]) {
                throw new Error(`Node does not have any credentials set for "${type}"!`);
            }
        }
        else {
            return undefined;
        }
    }
    const name = node.credentials[type];
    if (!additionalData.credentials[type]) {
        throw new Error(`No credentials of type "${type}" exist.`);
    }
    if (!additionalData.credentials[type][name]) {
        throw new Error(`No credentials with name "${name}" exist for type "${type}".`);
    }
    const credentialData = additionalData.credentials[type][name];
    const credentials = new _1.Credentials(name, type, credentialData.nodesAccess, credentialData.data);
    const decryptedDataObject = credentials.getData(additionalData.encryptionKey, node.type);
    if (decryptedDataObject === null) {
        throw new Error('Could not get the credentials');
    }
    return decryptedDataObject;
}
exports.getCredentials = getCredentials;
function getNodeParameter(workflow, runExecutionData, runIndex, connectionInputData, node, parameterName, itemIndex, fallbackValue) {
    const nodeType = workflow.nodeTypes.getByName(node.type);
    if (nodeType === undefined) {
        throw new Error(`Node type "${node.type}" is not known so can not return paramter value!`);
    }
    const value = lodash_1.get(node.parameters, parameterName, fallbackValue);
    if (value === undefined) {
        throw new Error(`Could not get parameter "${parameterName}"!`);
    }
    const returnData = workflow.getParameterValue(value, runExecutionData, runIndex, itemIndex, node.name, connectionInputData);
    return returnData;
}
exports.getNodeParameter = getNodeParameter;
function getNodeWebhookUrl(name, workflow, node, additionalData, isTest) {
    let baseUrl = additionalData.webhookBaseUrl;
    if (isTest === true) {
        baseUrl = additionalData.webhookTestBaseUrl;
    }
    const webhookDescription = getWebhookDescription(name, workflow, node);
    if (webhookDescription === undefined) {
        return undefined;
    }
    const path = workflow.getSimpleParameterValue(node, webhookDescription['path']);
    if (path === undefined) {
        return undefined;
    }
    return n8n_workflow_1.NodeHelpers.getNodeWebhookUrl(baseUrl, workflow.id, node, path.toString());
}
exports.getNodeWebhookUrl = getNodeWebhookUrl;
function getTimezone(workflow, additionalData) {
    if (workflow.settings !== undefined && workflow.settings.timezone !== undefined) {
        return workflow.settings.timezone;
    }
    return additionalData.timezone;
}
exports.getTimezone = getTimezone;
function getWebhookDescription(name, workflow, node) {
    const nodeType = workflow.nodeTypes.getByName(node.type);
    if (nodeType.description.webhooks === undefined) {
        return undefined;
    }
    for (const webhookDescription of nodeType.description.webhooks) {
        if (webhookDescription.name === name) {
            return webhookDescription;
        }
    }
    return undefined;
}
exports.getWebhookDescription = getWebhookDescription;
function getExecuteTriggerFunctions(workflow, node, additionalData, mode) {
    return ((workflow, node) => {
        return {
            emit: (data) => {
                throw new Error('Overwrite NodeExecuteFunctions.getExecuteTriggerFunctions.emit function!');
            },
            getCredentials(type) {
                return getCredentials(workflow, node, type, additionalData);
            },
            getMode: () => {
                return mode;
            },
            getNodeParameter: (parameterName, fallbackValue) => {
                const runExecutionData = null;
                const itemIndex = 0;
                const runIndex = 0;
                const connectionInputData = [];
                return getNodeParameter(workflow, runExecutionData, runIndex, connectionInputData, node, parameterName, itemIndex, fallbackValue);
            },
            getTimezone: () => {
                return getTimezone(workflow, additionalData);
            },
            getWorkflowStaticData(type) {
                return workflow.getStaticData(type, node);
            },
            helpers: {
                prepareBinaryData,
                request: requestPromise,
                returnJsonArray,
            },
        };
    })(workflow, node);
}
exports.getExecuteTriggerFunctions = getExecuteTriggerFunctions;
function getExecuteFunctions(workflow, runExecutionData, runIndex, connectionInputData, inputData, node, additionalData, mode) {
    return ((workflow, runExecutionData, connectionInputData, inputData, node) => {
        return {
            getContext(type) {
                return n8n_workflow_1.NodeHelpers.getContext(runExecutionData, type, node);
            },
            getCredentials(type) {
                return getCredentials(workflow, node, type, additionalData);
            },
            getInputData: (inputIndex = 0, inputName = 'main') => {
                if (!inputData.hasOwnProperty(inputName)) {
                    return [];
                }
                if (inputData[inputName].length < inputIndex) {
                    throw new Error(`Could not get input index "${inputIndex}" of input "${inputName}"!`);
                }
                if (inputData[inputName][inputIndex] === null) {
                    throw new Error(`Value "${inputIndex}" of input "${inputName}" did not get set!`);
                }
                return inputData[inputName][inputIndex];
            },
            getNodeParameter: (parameterName, itemIndex, fallbackValue) => {
                return getNodeParameter(workflow, runExecutionData, runIndex, connectionInputData, node, parameterName, itemIndex, fallbackValue);
            },
            getMode: () => {
                return mode;
            },
            getTimezone: () => {
                return getTimezone(workflow, additionalData);
            },
            getWorkflowStaticData(type) {
                return workflow.getStaticData(type, node);
            },
            prepareOutputData: n8n_workflow_1.NodeHelpers.prepareOutputData,
            helpers: {
                prepareBinaryData,
                request: requestPromise,
                returnJsonArray,
            },
        };
    })(workflow, runExecutionData, connectionInputData, inputData, node);
}
exports.getExecuteFunctions = getExecuteFunctions;
function getExecuteSingleFunctions(workflow, runExecutionData, runIndex, connectionInputData, inputData, node, itemIndex, additionalData, mode) {
    return ((workflow, runExecutionData, connectionInputData, inputData, node, itemIndex) => {
        return {
            getContext(type) {
                return n8n_workflow_1.NodeHelpers.getContext(runExecutionData, type, node);
            },
            getCredentials(type) {
                return getCredentials(workflow, node, type, additionalData);
            },
            getInputData: (inputIndex = 0, inputName = 'main') => {
                if (!inputData.hasOwnProperty(inputName)) {
                    return { json: {} };
                }
                if (inputData[inputName].length < inputIndex) {
                    throw new Error(`Could not get input index "${inputIndex}" of input "${inputName}"!`);
                }
                const allItems = inputData[inputName][inputIndex];
                if (allItems === null) {
                    throw new Error(`Value "${inputIndex}" of input "${inputName}" did not get set!`);
                }
                if (allItems[itemIndex] === null) {
                    throw new Error(`Value "${inputIndex}" of input "${inputName}" with itemIndex "${itemIndex}" did not get set!`);
                }
                return allItems[itemIndex];
            },
            getMode: () => {
                return mode;
            },
            getTimezone: () => {
                return getTimezone(workflow, additionalData);
            },
            getNodeParameter: (parameterName, fallbackValue) => {
                return getNodeParameter(workflow, runExecutionData, runIndex, connectionInputData, node, parameterName, itemIndex, fallbackValue);
            },
            getWorkflowStaticData(type) {
                return workflow.getStaticData(type, node);
            },
            helpers: {
                prepareBinaryData,
                request: requestPromise,
            },
        };
    })(workflow, runExecutionData, connectionInputData, inputData, node, itemIndex);
}
exports.getExecuteSingleFunctions = getExecuteSingleFunctions;
function getLoadOptionsFunctions(workflow, node, additionalData) {
    return ((workflow, node) => {
        const that = {
            getCredentials(type) {
                return getCredentials(workflow, node, type, additionalData);
            },
            getNodeParameter: (parameterName, fallbackValue) => {
                const runExecutionData = null;
                const itemIndex = 0;
                const runIndex = 0;
                const connectionInputData = [];
                return getNodeParameter(workflow, runExecutionData, runIndex, connectionInputData, node, parameterName, itemIndex, fallbackValue);
            },
            getTimezone: () => {
                return getTimezone(workflow, additionalData);
            },
            helpers: {
                request: requestPromise,
            },
        };
        return that;
    })(workflow, node);
}
exports.getLoadOptionsFunctions = getLoadOptionsFunctions;
function getExecuteHookFunctions(workflow, node, additionalData, mode, isTest, webhookData) {
    return ((workflow, node) => {
        const that = {
            getCredentials(type) {
                return getCredentials(workflow, node, type, additionalData);
            },
            getMode: () => {
                return mode;
            },
            getNodeParameter: (parameterName, fallbackValue) => {
                const runExecutionData = null;
                const itemIndex = 0;
                const runIndex = 0;
                const connectionInputData = [];
                return getNodeParameter(workflow, runExecutionData, runIndex, connectionInputData, node, parameterName, itemIndex, fallbackValue);
            },
            getNodeWebhookUrl: (name) => {
                return getNodeWebhookUrl(name, workflow, node, additionalData, isTest);
            },
            getTimezone: () => {
                return getTimezone(workflow, additionalData);
            },
            getWebhookName() {
                if (webhookData === undefined) {
                    throw new Error('Is only supported in webhook functions!');
                }
                return webhookData.webhookDescription.name;
            },
            getWebhookDescription(name) {
                return getWebhookDescription(name, workflow, node);
            },
            getWorkflowStaticData(type) {
                return workflow.getStaticData(type, node);
            },
            helpers: {
                request: requestPromise,
            },
        };
        return that;
    })(workflow, node);
}
exports.getExecuteHookFunctions = getExecuteHookFunctions;
function getExecuteWebhookFunctions(workflow, node, additionalData, mode, webhookData) {
    return ((workflow, node) => {
        return {
            getBodyData() {
                if (additionalData.httpRequest === undefined) {
                    throw new Error('Request is missing!');
                }
                return additionalData.httpRequest.body;
            },
            getCredentials(type) {
                return getCredentials(workflow, node, type, additionalData);
            },
            getHeaderData() {
                if (additionalData.httpRequest === undefined) {
                    throw new Error('Request is missing!');
                }
                return additionalData.httpRequest.headers;
            },
            getMode: () => {
                return mode;
            },
            getNodeParameter: (parameterName, fallbackValue) => {
                const runExecutionData = null;
                const itemIndex = 0;
                const runIndex = 0;
                const connectionInputData = [];
                return getNodeParameter(workflow, runExecutionData, runIndex, connectionInputData, node, parameterName, itemIndex, fallbackValue);
            },
            getQueryData() {
                if (additionalData.httpRequest === undefined) {
                    throw new Error('Request is missing!');
                }
                return additionalData.httpRequest.query;
            },
            getRequestObject() {
                if (additionalData.httpRequest === undefined) {
                    throw new Error('Request is missing!');
                }
                return additionalData.httpRequest;
            },
            getResponseObject() {
                if (additionalData.httpResponse === undefined) {
                    throw new Error('Response is missing!');
                }
                return additionalData.httpResponse;
            },
            getNodeWebhookUrl: (name) => {
                return getNodeWebhookUrl(name, workflow, node, additionalData);
            },
            getTimezone: () => {
                return getTimezone(workflow, additionalData);
            },
            getWorkflowStaticData(type) {
                return workflow.getStaticData(type, node);
            },
            getWebhookName() {
                return webhookData.webhookDescription.name;
            },
            prepareOutputData: n8n_workflow_1.NodeHelpers.prepareOutputData,
            helpers: {
                prepareBinaryData,
                request: requestPromise,
                returnJsonArray,
            },
        };
    })(workflow, node);
}
exports.getExecuteWebhookFunctions = getExecuteWebhookFunctions;
//# sourceMappingURL=NodeExecuteFunctions.js.map